/**
 * This interface is needed to bridge between different segment implementations
 * with Soya's ReduxStore implementation.
 *
 * Segment represents a segment of our unified store tree, specifically, a top
 * level key in our state tree:
 *
 * <pre>
 *   state = {
 *     segmentA: {...},
 *     segmentB: [...],
 *     ...
 *   };
 * </pre>
 *
 * IMPORTANT NOTE: The entirety of this class is supposed to be stateless.
 * The reason we use class is to make it easier to inherit some of the
 * pre-defined methods, while also making the interface clear.
 *
 * @CLIENT_SERVER
 */'use strict';Object.defineProperty(exports, '__esModule', { value: true });var _createClass = (function () {function defineProperties(target, props) {for (var i = 0; i < props.length; i++) {var descriptor = props[i];descriptor.enumerable = descriptor.enumerable || false;descriptor.configurable = true;if ('value' in descriptor) descriptor.writable = true;Object.defineProperty(target, descriptor.key, descriptor);}}return function (Constructor, protoProps, staticProps) {if (protoProps) defineProperties(Constructor.prototype, protoProps);if (staticProps) defineProperties(Constructor, staticProps);return Constructor;};})();function _classCallCheck(instance, Constructor) {if (!(instance instanceof Constructor)) {throw new TypeError('Cannot call a class as a function');}}var 
Segment = (function () {function Segment() {_classCallCheck(this, Segment);}_createClass(Segment, null, [{ key: 'getSegmentDependencies', 
    /**
     * Returns an array of Segment classes that this Segment has dependencies to.
     * Child classes can override this static method to declare their Segment
     * dependencies.
     *
     * @returns {Array<Class<Segment>}
     */value: 
    function getSegmentDependencies() {
      return [];}


    /**
     * Returns an array of Service classes that this Segment needs.
     *
     * @returns {Array<Class<Service>>}
     */ }, { key: 'getServiceDependencies', value: 
    function getServiceDependencies() {
      return [];}


    /**
     * Get the segment name. Segment's name is hard-coded by each segment
     * implementation and must never change.
     *
     * @return {string}
     */ }, { key: 'id', value: 
    function id() {
      throw new Error('Must implement: ' + this);}


    /**
     * Returns true if this segment requires hydration. Locally stored segments
     * will probably not require any hydration.
     *
     * @return {boolean}
     */ }, { key: 'shouldHydrate', value: 
    function shouldHydrate() {
      return true;}


    /**
     * The generated query ID needs to be string since it'll be used by ReduxStore
     * to store query-related data.
     *
     * @param {any} query
     * @return {string}
     */ }, { key: 'generateQueryId', value: 
    function generateQueryId(query) {
      throw new Error('Must implement: ' + this);}


    /**
     * Uses action creator to create load action of the given query.
     *
     * @param {any} query
     * @param {string} queryId
     * @param {any} segmentState
     * @param {{[key: string]: Service}} services
     * @return {void | Load}
     */ }, { key: 'createLoadFromQuery', value: 
    function createLoadFromQuery(query, queryId, segmentState, services) {
      return null;}


    /**
     * Returns an object that not only return the resulting data, but also whether
     * or not the data is loaded.
     * 
     * @param {any} query
     * @param {string} queryId
     * @param {any} segmentState
     * @return {QueryResult}
     */ }, { key: 'queryState', value: 
    function queryState(query, queryId, segmentState) {
      throw new Error('Must implement: ' + this);}


    /**
     * Compares two segment states, returns true if the segment state is
     * different.
     *
     * @param {any} segmentStateA
     * @param {any} segmentStateB
     * @return {boolean}
     */ }, { key: 'isStateEqual', value: 
    function isStateEqual(segmentStateA, segmentStateB) {
      return segmentStateA === segmentStateB;}


    /**
     * Compares pieces of two state. If they are equal return null, otherwise
     * return an array containing the current segment piece.
     *
     * The reason we enclose the piece in array is we need to differentiate
     * between equal values (null) and differing ones (array).
     *
     * @param prevSegmentState
     * @param segmentState
     * @param query
     * @param queryId
     * @return {?Array<any>}
     */ }, { key: 'comparePiece', value: 
    function comparePiece(prevSegmentState, segmentState, query, queryId) {
      // If state is equal, nothing has changed, since our reducer always
      // re-creates the object.
      if (this.isStateEqual(prevSegmentState, segmentState)) {
        return null;}


      var prevQueryResult = this.queryState(query, queryId, prevSegmentState);
      var queryResult = this.queryState(query, queryId, segmentState);
      if (prevQueryResult.loaded != queryResult.loaded) {
        return [queryResult.data];}


      var prevSegmentPiece = prevQueryResult.data;
      var segmentPiece = queryResult.data;
      if (segmentPiece === prevSegmentPiece) {
        return null;}


      return [segmentPiece];}


    /**
     * Returns a reducer function. Called only once by Store, on registration
     * of a new segment.
     *
     * Reducer is responsible for:
     * 1) Initializing default segment state.
     * 2) Handling changes to segment state.
     *
     * NOTE: We could easily convert this into reduce() method. However in the
     * spirit of redux, reducers should be stateless functions. Let state of
     * subscription, queries, etc. be handled by others. We're making it
     * much simpler this way, methinks.
     *
     * @return {Function}
     */ }, { key: 'getReducer', value: 
    function getReducer() {
      throw new Error('Must implement: ' + this);}


    /**
     * Returns an object containing action functions.
     *
     * @return {Object}
     */ }, { key: 'getActionCreator', value: 
    function getActionCreator() {
      throw new Error('Must implement: ' + this);}


    /**
     * Processes the refresh requests according to the given state and generates
     * list of queries that must be run with cache turned off.
     *
     * We need to have this method because we only wanted to refresh states that
     * are already loaded, not the
     *
     * @param {?} segmentState
     * @param {?} refreshRequests
     * @returns {Array<?>}
     */ }, { key: 'processRefreshRequests', value: 
    function processRefreshRequests(segmentState, refreshRequests) {
      return [];} }]);return Segment;})();exports['default'] = Segment;module.exports = exports['default'];
//# sourceMappingURL=Segment.js.map